using DotNext;

namespace Common
{
    class NumberProcessor
    {
        public static Result<byte[]> NumberToBytes(ulong num, uint length, bool isRightHigh = false)
        {
            if (length > 8)
            {
                throw new ArgumentException(
                    "Unsigned long number can't over 8 bytes(64 bits).",
                    nameof(length)
                );
            }

            var arr = new byte[length];

            if (isRightHigh)
            {
                for (var i = 0; i < length; i++)
                {
                    arr[length - 1 - i] = Convert.ToByte((num >> (i << 3)) & (0xFF));
                }
            }
            else
            {
                for (var i = 0; i < length; i++)
                {
                    arr[i] = Convert.ToByte((num >> ((int)(length - 1 - i) << 3)) & (0xFF));
                }
            }
            return arr;
        }

        public static Result<ulong> BytesToNumber(byte[] bytes, bool isRightHigh = false)
        {
            if (bytes.Length > 8)
            {
                throw new ArgumentException(
                    "Unsigned long number can't over 8 bytes(64 bits).",
                    nameof(bytes)
                );
            }

            ulong num = 0;
            int len = bytes.Length;

            if (isRightHigh)
            {
                for (var i = 0; i < len; i++)
                {
                    num += Convert.ToUInt64((UInt64)bytes[len - 1 - i] << (i << 3));
                }
            }
            else
            {
                for (var i = 0; i < len; i++)
                {
                    num += Convert.ToUInt64((UInt64)bytes[i] << ((int)(len - 1 - i) << 3));
                }
            }

            return num;
        }



        public static Result<byte[]> MultiBitsToBytes(ulong bits1, uint bits1Len, ulong bits2, uint bits2Len)
        {
            return NumberToBytes(MultiBitsToNumber(bits1, bits1Len, bits2, bits2Len).Value,
                        (bits1Len + bits2Len) % 8 != 0 ? (bits1Len + bits2Len) / 8 + 1 : (bits1Len + bits2Len) / 8);
        }

        public static Result<ulong> MultiBitsToNumber(ulong bits1, uint bits1Len, ulong bits2, uint bits2Len)
        {
            if (bits1Len + bits2Len > 64) throw new ArgumentException("Two Bits is more than 64 bits");

            ulong num = (bits1 << Convert.ToInt32(bits2Len)) | bits2;
            return num;
        }

        public static Result<uint> MultiBitsToNumber(uint bits1, uint bits1Len, uint bits2, uint bits2Len)
        {
            if (bits1Len + bits2Len > 64) throw new ArgumentException("Two Bits is more than 64 bits");

            uint num = (bits1 << Convert.ToInt32(bits2Len)) | bits2;
            return num;
        }

        public static bool BitsCheck(ulong srcBits, ulong dstBits, ulong mask = 0xFFFF_FFFF_FFFF_FFFF)
        {
            return (srcBits & mask) == dstBits;
        }

        public static bool BitsCheck(uint srcBits, uint dstBits, uint mask = 0xFFFF_FFFF)
        {
            return (srcBits & mask) == dstBits;
        }


        public static byte[] StringToBytes(string str, int numBase = 16)
        {
            var len = str.Length;
            var bytesLen = len / 2;
            var bytes = new byte[bytesLen];

            for (var i = 0; i < bytesLen; i++)
            {
                bytes[i] = Convert.ToByte(str.Substring(i * 2, 2), 16);
            }

            return bytes;
        }

    }

}
