using System.Net;
using System.Net.Sockets;
using System.Text;

class UDPClientPool
{
    private static IPAddress localhost = IPAddress.Parse("127.0.0.1");

    public static bool SendString(IPEndPoint endPoint, string[] stringArray)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);

        byte[] sendbuf = Encoding.ASCII.GetBytes(stringArray[0]);

        var sendLen = socket.SendTo(sendbuf, endPoint);
        if (sendLen == stringArray[0].Length) { return true; }
        else { return false; }
    }

    public async static ValueTask<bool> SendStringAsync(IPEndPoint endPoint, string[] stringArray)
    {
        return await Task.Run(() => { return SendString(endPoint, stringArray); });
    }

    public static bool SendBytes(IPEndPoint endPoint, byte[] buf)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        var sendLen = socket.SendTo(buf, endPoint);
        if (sendLen == buf.Length) { return true; }
        else { return false; }
    }

    public async static ValueTask<bool> SendBytesAsync(IPEndPoint endPoint, byte[] buf)
    {
        return await Task.Run(() => { return SendBytes(endPoint, buf); });
    }

    public static bool SendAddrPack(IPEndPoint endPoint, WebProtocol.SendAddrPackage pkg)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        var sendBytes = pkg.ToBytes();
        var sendLen = socket.SendTo(sendBytes, endPoint);
        if (sendLen == sendBytes.Length) { return true; }
        else { return false; }
    }

    public async static ValueTask<bool> SendAddrPackAsync(IPEndPoint endPoint, WebProtocol.SendAddrPackage pkg)
    {
        return await Task.Run(() => { return SendAddrPack(endPoint, pkg); });
    }

    public static bool SendDataPack(IPEndPoint endPoint, WebProtocol.SendDataPackage pkg)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        var sendBytes = pkg.ToBytes();
        var sendLen = socket.SendTo(sendBytes, endPoint);
        if (sendLen == sendBytes.Length) { return true; }
        else { return false; }
    }

    public async static ValueTask<bool> SendDataPackAsync(IPEndPoint endPoint, WebProtocol.SendDataPackage pkg)
    {
        return await Task.Run(() => { return SendDataPack(endPoint, pkg); });
    }

    public static bool SendLocalHost(int port, string[] stringArray)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);

        byte[] sendBytes = Encoding.ASCII.GetBytes(stringArray[0]);
        IPEndPoint ep = new IPEndPoint(localhost, port);

        var sendLen = socket.SendTo(sendBytes, ep);
        if (sendLen == sendBytes.Length) { return true; }
        else { return false; }
    }

    public static void CycleSendLocalHost(int times, int sleepMilliSeconds, int port, string[] stringArray)
    {
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        byte[] sendbuf = Encoding.ASCII.GetBytes(stringArray[0]);
        IPEndPoint ep = new IPEndPoint(localhost, port);

        while (times-- >= 0)
        {
            socket.SendTo(sendbuf, ep);

            Thread.Sleep(sleepMilliSeconds);
        }
    }
}
